@@ -17,7 +17,7 @@
    ;; primary-expression => constant
    (lambda ($1 . $rest) `(p-expr ,$1))
    ;; primary-expression => string-literal
-   (lambda ($1 . $rest) `(p-expr ,(tl->list $1)))
+   (lambda ($1 . $rest) `(p-expr ,$1))
    ;; primary-expression => "(" expression ")"
    (lambda ($3 $2 $1 . $rest) $2)
    ;; primary-expression => "(" "{" block-item-list "}" ")"
@@ -189,56 +189,65 @@
        `(comma-expr ,$1 ,$3)))
    ;; constant-expression => conditional-expression
    (lambda ($1 . $rest) $1)
-   ;; declaration => declaration-specifiers init-declarator-list opt-attrib...
-   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (if (pair? $3)
-       (if (pair? $6)
-         `(decl ,$3 ,(sx-ref $4 1) ,(sx-ref $4 2) ,$6)
-         `(decl ,$3 ,(sx-ref $4 1) ,(sx-ref $4 2)))
-       (if (pair? $6)
-         `(decl ,(sx-ref $4 1) ,(sx-ref $4 2) ,$6)
-         $4)))
-   ;; declaration => declaration-specifiers ";" opt-code-comment
-   (lambda ($3 $2 $1 . $rest)
-     (if (pair? $3)
-       `(decl ,(tl->list $1) ,(list $3))
-       `(decl ,(tl->list $1))))
-   ;; $P1 => 
-   (lambda ($3 $2 $1 . $rest)
-     (save-typenames
-       `(decl ,(tl->list $1) ,(tl->list $2))))
-   ;; declaration-specifiers => storage-class-specifier
+   ;; declaration => declaration-no-comment ";"
+   (lambda ($2 $1 . $rest) $1)
+   ;; declaration => declaration-no-comment ";" code-comment
+   (lambda ($3 $2 $1 . $rest) (append $1 (list $3)))
+   ;; declaration-no-comment => declaration-specifiers init-declarator-list
+   (lambda ($2 $1 . $rest)
+     (save-typenames `(decl ,$1 ,$2)))
+   ;; declaration-no-comment => declaration-specifiers
+   (lambda ($1 . $rest)
+     (if (only-attr-specs? $1)
+       `(expr-stmt)
+       `(decl ,$1)))
+   ;; declaration-specifiers => declaration-specifiers-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; declaration-specifiers-1 => storage-class-specifier
    (lambda ($1 . $rest)
      (make-tl 'decl-spec-list $1))
-   ;; declaration-specifiers => storage-class-specifier declaration-specifiers
+   ;; declaration-specifiers-1 => storage-class-specifier declaration-speci...
    (lambda ($2 $1 . $rest) (tl-insert $2 $1))
-   ;; declaration-specifiers => type-specifier
+   ;; declaration-specifiers-1 => type-specifier
    (lambda ($1 . $rest)
      (make-tl 'decl-spec-list $1))
-   ;; declaration-specifiers => type-specifier declaration-specifiers
+   ;; declaration-specifiers-1 => type-specifier declaration-specifiers-1
    (lambda ($2 $1 . $rest) (tl-insert $2 $1))
-   ;; declaration-specifiers => type-qualifier
+   ;; declaration-specifiers-1 => type-qualifier
    (lambda ($1 . $rest)
      (make-tl 'decl-spec-list $1))
-   ;; declaration-specifiers => type-qualifier declaration-specifiers
+   ;; declaration-specifiers-1 => type-qualifier declaration-specifiers-1
    (lambda ($2 $1 . $rest) (tl-insert $2 $1))
-   ;; declaration-specifiers => function-specifier
+   ;; declaration-specifiers-1 => function-specifier
    (lambda ($1 . $rest)
      (make-tl 'decl-spec-list $1))
-   ;; declaration-specifiers => function-specifier declaration-specifiers
+   ;; declaration-specifiers-1 => function-specifier declaration-specifiers-1
    (lambda ($2 $1 . $rest) (tl-insert $2 $1))
-   ;; init-declarator-list => init-declarator
+   ;; declaration-specifiers-1 => attribute-specifier declaration-specifiers-1
+   (lambda ($2 $1 . $rest) (tl-insert $2 $1))
+   ;; init-declarator-list => init-declarator-list-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; init-declarator-list-1 => init-declarator
    (lambda ($1 . $rest)
      (make-tl 'init-declr-list $1))
-   ;; init-declarator-list => init-declarator-list "," init-declarator
+   ;; init-declarator-list-1 => init-declarator-list-1 "," init-declarator
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
-   ;; init-declarator => declarator
+   ;; init-declarator => init-decl-no-attr
+   (lambda ($1 . $rest) $1)
+   ;; init-declarator => attribute-specifiers init-decl-no-attr
+   (lambda ($2 $1 . $rest) $1)
+   ;; init-decl-no-attr => declarator
    (lambda ($1 . $rest) `(init-declr ,$1))
-   ;; init-declarator => declarator asm-expression
-   (lambda ($2 $1 . $rest) `(init-declr ,$1))
-   ;; init-declarator => declarator "=" initializer
+   ;; init-decl-no-attr => declarator "=" initializer
    (lambda ($3 $2 $1 . $rest) `(init-declr ,$1 ,$3))
-   ;; init-declarator => declarator asm-expression "=" initializer
+   ;; init-decl-no-attr => declarator asm-expression
+   (lambda ($2 $1 . $rest) `(init-declr ,$1))
+   ;; init-decl-no-attr => declarator asm-expression "=" initializer
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(init-declr ,$1 ,$4))
+   ;; init-decl-no-attr => declarator attribute-specifiers
+   (lambda ($2 $1 . $rest) `(init-declr ,$1))
+   ;; init-decl-no-attr => declarator attribute-specifiers "=" initializer
    (lambda ($4 $3 $2 $1 . $rest)
      `(init-declr ,$1 ,$4))
    ;; storage-class-specifier => "auto"
@@ -357,106 +366,32 @@
    ;; complex-type-specifier => "long" "double" "_Complex"
    (lambda ($3 $2 $1 . $rest)
      '(complex-type "long double _Complex"))
-   ;; attribute-specifier => attribute-specifier-single
-   (lambda ($1 . $rest) $1)
-   ;; attribute-specifier => attribute-specifier attribute-specifier-single
-   (lambda ($2 $1 . $rest) (append $1 (cdr $2)))
-   ;; attribute-specifier-single => "__attribute__" "(" "(" attribute-list ...
+   ;; struct-or-union-specifier => "struct" opt-attr-specs ident-like "{" s...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (tl->list $4))
-   ;; attribute-specifier-single => attr-name
-   (lambda ($1 . $rest) `(attributes $1))
-   ;; attr-name => "__packed__"
-   (lambda ($1 . $rest) $1)
-   ;; attr-name => "__aligned__"
-   (lambda ($1 . $rest) $1)
-   ;; attr-name => "__alignof__"
-   (lambda ($1 . $rest) $1)
-   ;; attr-ident => attr-name
-   (lambda ($1 . $rest) $1)
-   ;; attr-ident => '$ident
-   (lambda ($1 . $rest) $1)
-   ;; attribute-list => 
-   (lambda $rest '(@))
-   ;; attribute-list => attribute
-   (lambda ($1 . $rest) (make-tl '@ $1))
-   ;; attribute-list => attribute-list ","
-   (lambda ($2 $1 . $rest) $1)
-   ;; attribute-list => attribute-list "," attribute
-   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
-   ;; attribute => attr-ident
-   (lambda ($1 . $rest)
-     (list (string->symbol $1) ""))
-   ;; attribute => attr-ident "(" attr-expr-list ")"
-   (lambda ($4 $3 $2 $1 . $rest)
-     (list (string->symbol $1)
-           (attr-expr-list->string (tl->list $3))))
-   ;; attr-expr-list => attribute-expr
-   (lambda ($1 . $rest)
-     (make-tl 'attr-expr-list $1))
-   ;; attr-expr-list => attr-expr-list "," attribute-expr
-   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
-   ;; attr-call-name => '$ident
-   (lambda ($1 . $rest) $1)
-   ;; attr-call-name => attr-name
-   (lambda ($1 . $rest) $1)
-   ;; attribute-expr => type-name
-   (lambda ($1 . $rest) (sx-ref* $1 1 1 1 1))
-   ;; attribute-expr => '$fixed
-   (lambda ($1 . $rest) $1)
-   ;; attribute-expr => '$string
-   (lambda ($1 . $rest) $1)
-   ;; attribute-expr => attr-call-name
-   (lambda ($1 . $rest) $1)
-   ;; attribute-expr => attr-call-name "(" attr-expr-list ")"
-   (lambda ($4 $3 $2 $1 . $rest)
-     (string-append
-       $1
-       (attr-expr-list->string (tl->list $3))))
-   ;; opt-attribute-specifier => 
-   (lambda $rest (list))
-   ;; opt-attribute-specifier => attribute-specifier
-   (lambda ($1 . $rest) $1)
-   ;; struct-or-union-specifier => "struct" ident-like "{" struct-declarati...
-   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (if (pair? $6)
-       `(struct-def ,$6 ,$2 ,(tl->list $4))
-       `(struct-def ,$2 ,(tl->list $4))))
-   ;; struct-or-union-specifier => "struct" "{" struct-declaration-list "}"...
+     (sx-join* 'struct-def $2 $3 (tl->list $5)))
+   ;; struct-or-union-specifier => "struct" opt-attr-specs "{" struct-decla...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     (if (pair? $5)
-       `(struct-def ,$5 ,(tl->list $3))
-       `(struct-def ,(tl->list $3))))
-   ;; struct-or-union-specifier => "struct" ident-like
-   (lambda ($2 $1 . $rest) `(struct-ref ,$2))
-   ;; struct-or-union-specifier => "union" ident-like "{" struct-declaratio...
+     (sx-join* 'struct-def $2 (tl->list $4)))
+   ;; struct-or-union-specifier => "struct" opt-attr-specs ident-like
+   (lambda ($3 $2 $1 . $rest)
+     (sx-join* 'struct-ref $1 $3))
+   ;; struct-or-union-specifier => "union" opt-attr-specs ident-like "{" st...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (if (pair? $6)
-       `(union-def ,$6 ,$2 ,(tl->list $4))
-       `(union-def ,$2 ,(tl->list $4))))
-   ;; struct-or-union-specifier => "union" "{" struct-declaration-list "}" ...
+     (sx-join* 'union-def $2 $3 (tl->list $5)))
+   ;; struct-or-union-specifier => "union" opt-attr-specs "{" struct-declar...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     (if (pair? $5)
-       `(union-def ,$5 ,(tl->list $3))
-       `(union-def ,(tl->list $3))))
-   ;; struct-or-union-specifier => "union" ident-like
-   (lambda ($2 $1 . $rest) `(union-ref ,$2))
-   ;; struct-or-union-specifier => "struct" attribute-specifier ident-like ...
-   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(struct-def ,$2 ,(tl->list $4)))
-   ;; struct-or-union-specifier => "struct" attribute-specifier "{" struct-...
-   (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(struct-def ,(tl->list $3)))
-   ;; struct-or-union-specifier => "union" attribute-specifier ident-like "...
-   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(union-def ,$2 ,(tl->list $4)))
-   ;; struct-or-union-specifier => "union" attribute-specifier "{" struct-d...
-   (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(union-def ,(tl->list $3)))
+     (sx-join* 'union-def $2 (tl->list $4)))
+   ;; struct-or-union-specifier => "union" opt-attr-specs ident-like
+   (lambda ($3 $2 $1 . $rest)
+     (sx-join* 'union-ref $2 $3))
    ;; ident-like => identifier
    (lambda ($1 . $rest) $1)
    ;; ident-like => typedef-name
    (lambda ($1 . $rest) `(ident ,(sx-ref $1 1)))
+   ;; opt-attr-specs => 
+   (lambda $rest (list))
+   ;; opt-attr-specs => attribute-specifiers
+   (lambda ($1 . $rest) (attr-spec->attr $1))
    ;; struct-declaration-list => struct-declaration
    (lambda ($1 . $rest) (make-tl 'field-list $1))
    ;; struct-declaration-list => lone-comment
@@ -469,20 +404,15 @@
    (lambda ($1 . $rest) (make-tl 'field-list))
    ;; struct-declaration-list => struct-declaration-list ";"
    (lambda ($2 $1 . $rest) $1)
-   ;; struct-declaration => specifier-qualifier-list struct-declarator-list...
-   (lambda ($5 $4 $3 $2 $1 . $rest)
-     (if (pair? $3)
-       (if (pair? $5)
-         `(comp-decl ,$3 ,(tl->list $1) ,(tl->list $2) ,$5)
-         `(comp-decl ,$3 ,(tl->list $1) ,(tl->list $2)))
-       (if (pair? $5)
-         `(comp-decl ,(tl->list $1) ,(tl->list $2) ,$5)
-         `(comp-decl ,(tl->list $1) ,(tl->list $2)))))
-   ;; struct-declaration => specifier-qualifier-list ";" opt-code-comment
-   (lambda ($3 $2 $1 . $rest)
-     (if (pair? $3)
-       `(comp-decl ,(tl->list $1) ,$3)
-       `(comp-decl ,(tl->list $1))))
+   ;; struct-declaration => struct-declaration-no-comment ";"
+   (lambda ($2 $1 . $rest) $1)
+   ;; struct-declaration => struct-declaration-no-comment ";" code-comment
+   (lambda ($3 $2 $1 . $rest) (append $1 (list $3)))
+   ;; struct-declaration-no-comment => specifier-qualifier-list struct-decl...
+   (lambda ($2 $1 . $rest)
+     `(comp-decl ,(tl->list $1) ,(tl->list $2)))
+   ;; struct-declaration-no-comment => specifier-qualifier-list
+   (lambda ($1 . $rest) `(comp-decl ,(tl->list $1)))
    ;; specifier-qualifier-list => type-specifier specifier-qualifier-list
    (lambda ($2 $1 . $rest) (tl-insert $2 $1))
    ;; specifier-qualifier-list => type-specifier
@@ -500,6 +430,8 @@
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; struct-declarator => declarator
    (lambda ($1 . $rest) `(comp-declr ,$1))
+   ;; struct-declarator => declarator attribute-specifiers
+   (lambda ($2 $1 . $rest) `(comp-declr ,$1))
    ;; struct-declarator => declarator ":" constant-expression
    (lambda ($3 $2 $1 . $rest)
      `(comp-declr (bit-field ,$1 ,$3)))
@@ -526,6 +458,8 @@
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; enumerator => identifier
    (lambda ($1 . $rest) `(enum-defn ,$1))
+   ;; enumerator => identifier attribute-specifiers
+   (lambda ($2 $1 . $rest) `(enum-defn ,$1))
    ;; enumerator => identifier "=" constant-expression
    (lambda ($3 $2 $1 . $rest) `(enum-defn ,$1 ,$3))
    ;; type-qualifier => "const"
@@ -538,14 +472,73 @@
    (lambda ($1 . $rest) `(fctn-spec ,$1))
    ;; function-specifier => "_Noreturn"
    (lambda ($1 . $rest) `(fctn-spec ,$1))
+   ;; attribute-specifiers => attribute-specifier
+   (lambda ($1 . $rest) $1)
+   ;; attribute-specifiers => attribute-specifiers attribute-specifier
+   (lambda ($2 $1 . $rest) (append $1 (cdr $2)))
+   ;; attribute-specifier => "__attribute__" "(" "(" attribute-list ")" ")"
+   (lambda ($6 $5 $4 $3 $2 $1 . $rest)
+     (tl->list $4))
+   ;; attribute-specifier => attr-name
+   (lambda ($1 . $rest) `(attributes ,$1))
+   ;; attr-name => "__packed__"
+   (lambda ($1 . $rest) '(ident "packed"))
+   ;; attr-name => "__aligned__"
+   (lambda ($1 . $rest) '(ident "aligned"))
+   ;; attr-name => "__alignof__"
+   (lambda ($1 . $rest) '(ident "alignof"))
+   ;; attribute-list => attribute
+   (lambda ($1 . $rest) (make-tl 'attributes $1))
+   ;; attribute-list => attribute-list "," attribute
+   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
+   ;; attribute-list => attribute-list ","
+   (lambda ($2 $1 . $rest) $1)
+   ;; attribute => attr-word
+   (lambda ($1 . $rest) `(attribute ,$1))
+   ;; attribute => attr-word "(" attr-expr-list ")"
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(attribute ,$1 ,$3))
+   ;; attr-word => attr-name
+   (lambda ($1 . $rest) $1)
+   ;; attr-word => identifier
+   (lambda ($1 . $rest) $1)
+   ;; attr-expr-list => attr-expr-list-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; attr-expr-list-1 => attribute-expr
+   (lambda ($1 . $rest)
+     (make-tl 'attr-expr-list $1))
+   ;; attr-expr-list-1 => attr-expr-list-1 "," attribute-expr
+   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
+   ;; attribute-expr => type-name
+   (lambda ($1 . $rest) $1)
+   ;; attribute-expr => '$fixed
+   (lambda ($1 . $rest) `(fixed ,$1))
+   ;; attribute-expr => string-literal
+   (lambda ($1 . $rest) (join-string-literal $1))
+   ;; attribute-expr => identifier
+   (lambda ($1 . $rest) $1)
+   ;; attribute-expr => attr-word "(" attr-expr-list ")"
+   (lambda ($4 $3 $2 $1 . $rest) `(ident "FOO"))
    ;; declarator => pointer direct-declarator
    (lambda ($2 $1 . $rest) `(ptr-declr ,$1 ,$2))
    ;; declarator => direct-declarator
    (lambda ($1 . $rest) $1)
+   ;; pointer => "*" type-qualifier-list pointer
+   (lambda ($3 $2 $1 . $rest) `(pointer ,$2 ,$3))
+   ;; pointer => "*" type-qualifier-list
+   (lambda ($2 $1 . $rest) `(pointer ,$2))
+   ;; pointer => "*" pointer
+   (lambda ($2 $1 . $rest) `(pointer ,$2))
+   ;; pointer => "*" attribute-specifiers pointer
+   (lambda ($3 $2 $1 . $rest) `(pointer ,$3))
+   ;; pointer => "*"
+   (lambda ($1 . $rest) '(pointer))
    ;; direct-declarator => identifier
    (lambda ($1 . $rest) $1)
    ;; direct-declarator => "(" declarator ")"
    (lambda ($3 $2 $1 . $rest) `(scope ,$2))
+   ;; direct-declarator => "(" attribute-specifier declarator ")"
+   (lambda ($4 $3 $2 $1 . $rest) `(scope ,$2))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(array-of ,$1 ,$3 ,$4))
@@ -559,16 +552,10 @@
    (lambda ($3 $2 $1 . $rest) `(array-of ,$1))
    ;; direct-declarator => direct-declarator "[" "static" type-qualifier-li...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(array-of
-        ,$1
-        ,(tl->list (tl-insert '(stor-spec "static") $4))
-        ,$5))
+     `(array-of ,$1 ,$4 ,$5))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list "stati...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(array-of
-        ,$1
-        ,(tl->list (tl-insert '(stor-spec "static") $3))
-        ,$5))
+     `(array-of ,$1 ,4 ,$5))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list "*" "]"
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(array-of ,$1 ,$3 (var-len)))
@@ -584,20 +571,12 @@
    ;; direct-declarator => direct-declarator "(" ")"
    (lambda ($3 $2 $1 . $rest)
      `(ftn-declr ,$1 (param-list)))
-   ;; pointer => "*" type-qualifier-list pointer
-   (lambda ($3 $2 $1 . $rest)
-     `(pointer ,(tl->list $2) ,$3))
-   ;; pointer => "*" type-qualifier-list
-   (lambda ($2 $1 . $rest)
-     `(pointer ,(tl->list $2)))
-   ;; pointer => "*" pointer
-   (lambda ($2 $1 . $rest) `(pointer ,$2))
-   ;; pointer => "*"
-   (lambda ($1 . $rest) '(pointer))
-   ;; type-qualifier-list => type-qualifier
+   ;; type-qualifier-list => type-qualifier-list-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; type-qualifier-list-1 => type-qualifier
    (lambda ($1 . $rest)
      (make-tl 'type-qual-list $1))
-   ;; type-qualifier-list => type-qualifier-list type-qualifier
+   ;; type-qualifier-list-1 => type-qualifier-list-1 type-qualifier
    (lambda ($2 $1 . $rest) (tl-append $1 $2))
    ;; parameter-type-list => parameter-list
    (lambda ($1 . $rest) $1)
@@ -610,13 +589,12 @@
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; parameter-declaration => declaration-specifiers declarator
    (lambda ($2 $1 . $rest)
-     `(param-decl ,(tl->list $1) (param-declr ,$2)))
+     `(param-decl ,$1 (param-declr ,$2)))
    ;; parameter-declaration => declaration-specifiers abstract-declarator
    (lambda ($2 $1 . $rest)
-     `(param-decl ,(tl->list $1) (param-declr ,$2)))
+     `(param-decl ,$1 (param-declr ,$2)))
    ;; parameter-declaration => declaration-specifiers
-   (lambda ($1 . $rest)
-     `(param-decl ,(tl->list $1)))
+   (lambda ($1 . $rest) `(param-decl ,$1))
    ;; identifier-list => identifier
    (lambda ($1 . $rest) (make-tl 'ident-list $1))
    ;; identifier-list => identifier-list "," identifier
@@ -625,7 +603,7 @@
    (lambda ($2 $1 . $rest)
      `(type-name ,(tl->list $1) ,$2))
    ;; type-name => declaration-specifiers
-   (lambda ($1 . $rest) `(type-name ,(tl->list $1)))
+   (lambda ($1 . $rest) `(type-name ,$1))
    ;; abstract-declarator => pointer direct-abstract-declarator
    (lambda ($2 $1 . $rest) `(abs-declr ,$1 ,$2))
    ;; abstract-declarator => pointer
@@ -649,18 +627,18 @@
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
      `(declr-array
         ,$1
-        ,(tl->list (tl-insert '(stor-spec "static") $4))
+        ,(tl->list (tl-insert $4 '(stor-spec "static")))
         ,$5))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" "static"...
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(declr-array
         ,$1
-        ,(tl->list (tl-insert '(stor-spec "static") $4))))
+        ,(tl->list (tl-insert $4 '(stor-spec "static")))))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" type-qua...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
      `(declr-array
         ,$1
-        ,(tl->list (tl-insert '(stor-spec "static") $3))
+        ,(tl->list (tl-insert $3 '(stor-spec "static")))
         ,$5))
    ;; direct-abstract-declarator => "[" type-qualifier-list assignment-expr...
    (lambda ($4 $3 $2 $1 . $rest)
@@ -676,16 +654,16 @@
    ;; direct-abstract-declarator => "[" "static" type-qualifier-list assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(declr-anon-array
-        ,(tl->list (tl-insert '(stor-spec "static") $3))
+        ,(tl->list (tl-insert $3 '(stor-spec "static")))
         ,$4))
    ;; direct-abstract-declarator => "[" "static" type-qualifier-list "]"
    (lambda ($4 $3 $2 $1 . $rest)
      `(declr-anon-array
-        ,(tl->list (tl-insert '(stor-spec "static") $3))))
+        ,(tl->list (tl-insert $3 '(stor-spec "static")))))
    ;; direct-abstract-declarator => "[" type-qualifier-list "static" assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
      `(declr-anon-array
-        ,(tl->list (tl-insert '(stor-spec "static") $2))
+        ,(tl->list (tl-insert $2 '(stor-spec "static")))
         ,$4))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" "*" "]"
    (lambda ($4 $3 $2 $1 . $rest) `(declr-star ,$1))
@@ -750,6 +728,9 @@
    ;; labeled-statement => identifier ":" statement
    (lambda ($3 $2 $1 . $rest)
      `(labeled-stmt ,$1 ,$3))
+   ;; labeled-statement => identifier ":" attribute-specifier statement
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(labeled-stmt ,$1 ,$4))
    ;; labeled-statement => "case" constant-expression ":" statement
    (lambda ($4 $3 $2 $1 . $rest) `(case ,$2 ,$4))
    ;; labeled-statement => "default" ":" statement
@@ -865,9 +846,10 @@
    (lambda ($1 . $rest) (make-tl 'asm-clobbers))
    ;; asm-clobbers => ":" string-literal
    (lambda ($2 $1 . $rest)
-     (make-tl 'asm-clobbers $2))
+     (tl-extend (make-tl 'asm-clobbers) $2))
    ;; asm-clobbers => asm-clobbers "," string-literal
-   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
+   (lambda ($3 $2 $1 . $rest)
+     (tl-extend $1 (cdr $3)))
    ;; translation-unit => external-declaration-list
    (lambda ($1 . $rest) (tl->list $1))
    ;; external-declaration-list => 
@@ -881,15 +863,13 @@
    (lambda ($1 . $rest) $1)
    ;; external-declaration => declaration
    (lambda ($1 . $rest) $1)
-   ;; external-declaration => attribute-specifier declaration
-   (lambda ($2 $1 . $rest) $2)
    ;; external-declaration => lone-comment
    (lambda ($1 . $rest) $1)
    ;; external-declaration => cpp-statement
    (lambda ($1 . $rest) $1)
    ;; external-declaration => pragma
    (lambda ($1 . $rest) $1)
-   ;; external-declaration => "extern" '$string "{" $P2 external-declaratio...
+   ;; external-declaration => "extern" '$string "{" $P1 external-declaratio...
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
      `(extern-block
         (extern-begin ,$2)
@@ -898,9 +878,9 @@
    ;; external-declaration => ";"
    (lambda ($1 . $rest)
      `(decl (@ (extension "GNUC"))))
-   ;; $P2 => 
+   ;; $P1 => 
    (lambda ($3 $2 $1 . $rest) (cpi-dec-blev!))
-   ;; $P3 => 
+   ;; $P2 => 
    (lambda ($5 $4 $3 $2 $1 . $rest) (cpi-inc-blev!))
    ;; function-definition => declaration-specifiers declarator declaration-...
    (lambda ($4 $3 $2 $1 . $rest)
@@ -911,19 +891,13 @@
         ,$4))
    ;; function-definition => declaration-specifiers declarator compound-sta...
    (lambda ($3 $2 $1 . $rest)
-     `(fctn-defn ,(tl->list $1) ,$2 ,$3))
+     `(fctn-defn ,$1 ,$2 ,$3))
    ;; declaration-list => declaration
    (lambda ($1 . $rest) (make-tl $1))
    ;; declaration-list => declaration-list declaration
    (lambda ($2 $1 . $rest) (tl-append $1 $2))
-   ;; opt-code-comment => 
-   (lambda $rest (list))
-   ;; opt-code-comment => code-comment
-   (lambda ($1 . $rest) $1)
    ;; identifier => '$ident
    (lambda ($1 . $rest) `(ident ,$1))
-   ;; identifier => 'cpp-ident
-   (lambda ($1 . $rest) `(ident ,$1))
    ;; constant => '$fixed
    (lambda ($1 . $rest) `(fixed ,$1))
    ;; constant => '$float
@@ -939,9 +913,11 @@
    ;; constant => '$chlit/U
    (lambda ($1 . $rest)
      `(char (@ (type "char32_t")) ,$1))
-   ;; string-literal => '$string
+   ;; string-literal => string-literal-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; string-literal-1 => '$string
    (lambda ($1 . $rest) (make-tl 'string $1))
-   ;; string-literal => string-literal '$string
+   ;; string-literal-1 => string-literal-1 '$string
    (lambda ($2 $1 . $rest) (tl-append $1 $2))
    ;; code-comment => '$code-comm
    (lambda ($1 . $rest) `(comment ,$1))
@@ -950,8 +926,7 @@
    ;; cpp-statement => 'cpp-stmt
    (lambda ($1 . $rest) `(cpp-stmt ,$1))
    ;; pragma => "_Pragma" "(" string-literal ")"
-   (lambda ($4 $3 $2 $1 . $rest)
-     `(pragma ,(tl->list $3)))
+   (lambda ($4 $3 $2 $1 . $rest) `(pragma ,$3))
    ))
 
 ;;; end tables
