@@ -30,7 +30,7 @@
 ;; NOTE
 ;;  stripdown no longer deals with typeref expansion
 ;; use
-;;  expand-typerefs, then stripdown, then udecl->mspec
+;;  expand-typerefs, then stripdown, then udecl->mdecl
 ;;
 ;; NOTE
 ;;  unitize-decl is shallow: it only works on init-decl-list.
@@ -42,7 +42,7 @@
 ;;  if fixed/float use `int *x;' etc
 ;;  if function use `void *x;'
 
-;; mspec == munged (unwrapped) declaration
+;; mdecl == munged (unwrapped) declaration
 
 ;; possible pattern for munging:
 ;; (split-<form> form) => (values tag attr|#f ... tail)
@@ -71,7 +71,7 @@
 	    expand-typerefs
 	    stripdown
 	    udecl-rem-type-qual specl-rem-type-qual
-	    udecl->mspec udecl->mspec/comm mspec->udecl
+	    udecl->mdecl udecl->mdecl/comm mdecl->udecl
 
 	    unwrap-decl
 	    canize-enum-def-list
@@ -90,6 +90,7 @@
 	    inc-keeper?
 
 	    ;; deprecated
+	    udecl->mspec udecl->mspec/comm mspec->udecl
 	    tree->udict tree->udict/deep
 	    declr->ident
 	    match-decl match-comp-decl match-param-decl
@@ -199,7 +200,7 @@
 ;; @example
 ;; @end example
 
-;; mspec is
+;; mdecl is
 ;; ("foo" (pointer-to) (array-of 3) (fixed-type "unsigned int"))
 ;; which can be converted to
 ;; ("(*foo) (array-of 3) (fixed-type "unsigned int"))
@@ -1312,8 +1313,8 @@
 
 ;; === munged specification ============
 
-;; @deffn {Procedure} udecl->mspec udecl [#:abs-ident #f]
-;; @deffnx {Procedure} udecl->mspec/comm udecl [#:def-comm ""]
+;; @deffn {Procedure} udecl->mdecl udecl [#:abs-ident #f]
+;; @deffnx {Procedure} udecl->mdecl/comm udecl [#:def-comm ""]
 ;; Turn a stripped-down unit-declaration into an m-spec.  The second version
 ;; include a comment. This assumes decls have been run through
 ;; @code{stripdown}.
@@ -1329,7 +1330,7 @@
 ;; to add for abstract declarators.  If an identifier is not provided, a
 ;; random identifier starting with @code{@} will be provided.
 ;; @end deffn
-(define* (udecl->mspec decl #:key abs-ident)
+(define* (udecl->mdecl decl #:key abs-ident)
 
   ;; Hmm.  We convert array size back to C code (string).  Now that I am working
   ;; on constant expression eval (eval-c99-cx) maybe we should change that.
@@ -1422,7 +1423,7 @@
       (else
        (sferr "munge/unwrap-declr missed:\n")
        (pperr declr)
-       (error "c99/munge: udecl->mspec failed")
+       (error "c99/munge: udecl->mdecl failed")
        #f)))
 
   ;;(sferr "decl:\n") (pperr decl)
@@ -1439,42 +1440,42 @@
 	 (m-decl (reverse (cons m-specl m-declr))))
     ;;(sferr "decl:\n") (pperr decl)
     ;;(sferr "declr:\n") (pperr declr)
-    ;;(sferr "r-mspec: ~S\n" (cons m-specl m-declr))
+    ;;(sferr "r-mdecl: ~S\n" (cons m-specl m-declr))
     m-decl))
 
-(define* (udecl->mspec/comm decl #:key (def-comm ""))
+(define* (udecl->mdecl/comm decl #:key (def-comm ""))
   (let* ((comm (or (and=> (sx-ref decl 3) cadr) def-comm))
-	 (spec (udecl->mspec decl)))
+	 (spec (udecl->mdecl decl)))
     (cons* (car spec) comm (cdr spec))))
 
 
-;; @deffn {Procedure} mspec->udecl mspec xxx
+;; @deffn {Procedure} mdecl->udecl mdecl xxx
 ;; needed for xxx
 ;; @end deffn
-(define* (mspec->udecl mspec)
+(define* (mdecl->udecl mdecl)
 
   (define (make-udecl types declr)
     ;; TODO: w/ attr needed?
     `(udecl (decl-spec-list (type-spec ,types)) (init-declr ,declr)))
 
-  (define (doit declr mspec-tail)
-    (pmatch mspec-tail
-      (((fixed-type ,name)) (make-udecl (car mspec-tail) declr))
-      (((float-type ,name)) (make-udecl (car mspec-tail) declr))
-      (((typename ,name)) (make-udecl (car mspec-tail) declr))
-      (((void)) (make-udecl (car mspec-tail) declr))
+  (define (doit declr mdecl-tail)
+    (pmatch mdecl-tail
+      (((fixed-type ,name)) (make-udecl (car mdecl-tail) declr))
+      (((float-type ,name)) (make-udecl (car mdecl-tail) declr))
+      (((typename ,name)) (make-udecl (car mdecl-tail) declr))
+      (((void)) (make-udecl (car mdecl-tail) declr))
       
       (((pointer-to) . ,rest)
        (doit `(ptr-declr (pointer) ,declr) rest))
       
       (,*
-       (sferr "munge/mspec->udecl missed:\n")
-       (pperr mspec-tail)
-       (error "munge/mspec->udecl failed")
+       (sferr "munge/mdecl->udecl missed:\n")
+       (pperr mdecl-tail)
+       (error "munge/mdecl->udecl failed")
        #f)))
 
-  (let ((name (car mspec))
-	(rest (cdr mspec)))
+  (let ((name (car mdecl))
+	(rest (cdr mdecl)))
     (doit `(ident ,name) rest)))
 
 ;; === deprecated ====================
@@ -1488,6 +1489,9 @@
 (define declr->ident declr-ident)
 (define (fix-fields flds) (cdr (clean-field-list `(field-list . ,flds))))
 (define declr-is-ptr? pointer-declr?)
+(define udecl->mspec udecl->mdecl)
+(define udecl->mspec/comm udecl->mdecl/comm)
+(define mspec->udecl mdecl->udecl)
 
 ;;@deffn {Procedure} stripdown-1 udecl decl-dict [options]=> decl
 ;; This is deprecated.
@@ -1529,5 +1533,4 @@
 	 (specl1 (foldts fsD fsU fsH '() specl)))
     (list tag specl1 declr)))
 
-
 ;; --- last line ---
