@@ -19,10 +19,10 @@
 
 (define-module (nyacc lang sx-util)
   #:export (make-sx
-	    sx-tag sx-attr sx-tail sx-length sx-ref sx-ref* sx-cons* sx-list
+	    sx-tag sx-attr sx-tail sx-length sx-ref sx-ref*
 	    sx-has-attr? sx-attr-ref sx-attr-add sx-attr-add* sx-attr-set!
 	    sx-find
-	    sx-split sx-split* sx-join sx-join*
+	    sx-split sx-split* sx-join sx-join* sx-cons* sx-list
 	    sx-match)
   #:use-module ((srfi srfi-1) #:select (find fold fold-right)))
 (cond-expand
@@ -80,6 +80,12 @@
       ((eq? '@ (caadr sx)) (1- ln))
       (else ln))))
 
+;; @deffn {Procedure} sx-tag sx => tag
+;; Return the tag for a tree
+;; @end deffn
+(define (sx-tag sx)
+  (if (pair? sx) (car sx) #f))
+
 ;; @deffn {Procedure} sx-ref sx ix => item
 ;; Reference the @code{ix}-th element of the list, not counting the optional
 ;; attributes item.  If the list is shorter than the index, return @code{#f}.
@@ -111,57 +117,6 @@
 (define (sx-ref* sx . args)
   (fold (lambda (ix sx) (and (pair? sx) (sx-ref sx ix))) sx args))
 
-;; @deffn {Procedure} sx-tag sx => tag
-;; Return the tag for a tree
-;; @end deffn
-(define (sx-tag sx)
-  (if (pair? sx) (car sx) #f))
-
-;; @deffn {Procedure} sx-cons* tag (attr|#f)? ... => sx
-;; @deffnx {Procedure} sx-list tag (attr|#f)? ... => sx
-;; Generate the tag and the attr list if it exists.  Note that
-;; The following are equivalent:
-;; @example
-;; (sx-cons* tag attr elt1 elt2 '())
-;; (sx-list tag attr elt1 elt2)
-;; @end example
-;; @noindent
-;; NEW IMPLEMENTATION: now any field that is #f will be skipped
-;; @end deffn
-(define (sx-cons* tag . rest)
-  (if (null? rest) (error "expecing tail"))
-  ;;(cond
-  ;; ((null? rest) (list tag))
-  ;; ((not (car rest)) (apply cons* tag (cdr rest)))
-  ;; (else (apply cons* tag rest)))
-  (cons tag
-	(let iter ((items rest))
-	  (if (null? (cdr items)) (car items)
-	      (if (car items)
-		  (cons (car items) (iter (cdr items)))
-		  (iter (cdr items)))))))
-
-(define (sx-list tag . rest)
-  ;;(cond
-  ;; ((null? rest) (list tag))
-  ;; ((not (car rest)) (apply list tag (cdr rest)))
-  ;; (else (apply list tag rest))))
-  (cons tag
-	(let iter ((items rest))
-	  (if (null? items) '()
-	      (if (car items)
-		  (cons (car items) (iter (cdr items)))
-		  (iter (cdr items)))))))
-
-;;. maybe change to case-lambda to accept count
-;; @example
-;; (sx-repl-tail (tag (@ ...) (orig-elt ...)) (repl-elt ...))
-;; => 
-;; (sx-repl-tail (tag (@ ...) (repl-elt ...)))
-;; @end example
-(define (sx-repl-tail sexp tail)
-  (sx-cons* (sx-tag sexp) (sx-attr sexp) tail))
-
 ;; @deffn {Procedure} sx-tail sx [ix] => (list)
 ;; Return the ix-th tail starting after the tag and attribut list, where
 ;; @var{ix} must be positive.  For example,
@@ -181,78 +136,6 @@
    ((sx)
     (sx-tail sx 1))))
 
-;; @deffn {Procedure} sx-has-attr? sx
-;; p to determine if @arg{sx} has attributes.
-;; @end deffn
-(define (sx-has-attr? sx)
-  (and (pair? (cdr sx)) (pair? (cadr sx)) (eqv? '@ (caadr sx)) #t))
-
-;; @deffn {Procedure} sx-attr sx => '(@ ...)|#f
-;; @example
-;; (sx-attr '(abc (@ (foo "1")) def) 1) => '(@ (foo "1"))
-;; @end example
-;; should change this to
-;; @example
-;; (sx-attr sx) => '((a . 1) (b . 2) ...)
-;; @end example
-;; @end deffn
-(define (sx-attr sx)
-  (if (and (pair? (cdr sx)) (pair? (cadr sx)))
-      (if (eqv? '@ (caadr sx))
-	  (cadr sx)
-	  #f)
-      #f))
-
-;; @deffn {Procedure} sx-attr-ref sx|node|tail key => val
-;; Return an attribute value given the key, or @code{#f}.
-;; Also works if passed the attribute node @code{(@ ...)} or its tail.
-;; @end deffn
-(define (sx-attr-ref sx key)
-  (let ((attr-tail (cond ((null? sx) sx)
-			 ((pair? (car sx)) sx)
-			 ((eqv? '@ (car sx)) (car sx))
-			 ((sx-attr sx))
-			 (else '()))))
-    (and=> (assq-ref attr-tail key) car)))
-
-;; @deffn {Procedure} sx-attr-add sx key val
-;; Add attribute to sx.
-;; @end deffn
-(define (sx-attr-add sx key val)
-  (cons
-   (car sx)
-   (if (sx-has-attr? sx)
-       (cons `(@ (,key ,val)
-		 ,@(let iter ((atl (cdr (sx-attr sx))))
-		     (cond ((null? atl) '())
-			   ((eq? key (caar atl)) (iter (cdr atl)))
-			   (else (cons (car atl) (iter (cdr atl)))))))
-	     (cddr sx))
-       (cons `(@ (,key ,val)) (cdr sx)))))
-
-;; @deffn {Procedure} sx-attr-add* sx key val [key val [@dots{} ]] => sx
-;; Add key-val pairs. @var{key} must be a symbol and @var{val} must be
-;; a string.  Return a new @emph{sx}.
-;; @end deffn
-(define (sx-attr-add* sx . rest)
-  (let* ((attrs (if (sx-has-attr? sx) (cdr (sx-attr sx)) '()))
-	 (attrs (let iter ((kvl rest))
-		  (if (null? kvl) attrs
-		      (cons (list (car kvl) (cadr kvl)) (iter (cddr kvl)))))))
-    (cons* (sx-tag sx) (cons '@ attrs)
-	   (if (sx-has-attr? sx) (cddr sx) (cdr sx)))))
-
-;; @deffn {Procedure} sx-attr-set! sx key val
-;; Set attribute for sx.  If no attributes exist, if key does not exist,
-;; add it, if it does exist, replace it.
-;; @end deffn
-(define (sx-attr-set! sx key val)
-  (if (sx-has-attr? sx)
-      (let ((attr (cadr sx)))
-	(set-cdr! attr (assoc-set! (cdr attr) key (list val))))
-      (set-cdr! sx (cons `(@ (,key ,val)) (cdr sx))))
-  sx)
-
 ;; @deffn {Procedure} sx-find tag sx => (tag ...)
 ;; @deffnx {Procedure} sx-find path sx => (tag ...)
 ;; In the first form @var{tag} is a symbolic tag in the first level.
@@ -272,29 +155,152 @@
    (else
     (error "expecting first arg to be tag or sxpath"))))
 
-;; @deffn {Procedure} sx-split sexp => tag attr|#f tail
-;; @deffnx {Procedure} sx-split* sexp => tag attr|#f exp ...
-;; Split an SXML element into its constituent parts, as a @code{values}.
+;; @deffn {Procedure} sx-has-attr? sx
+;; p to determine if @arg{sx} has attributes.
+;; @end deffn
+(define (sx-has-attr? sx)
+  (and (pair? (cdr sx)) (pair? (cadr sx)) (eqv? '@ (caadr sx)) #t))
+
+;; @deffn {Procedure} sx-attr sx => ((k v) ...)
+;; @example
+;; (sx-attr '(abc (@ (foo "1")) def) 1) => ((foo "1"))
+;; @end example
+;; @end deffn
+(define (sx-attr sx)
+  (if (and (pair? (cdr sx))
+	   (pair? (cadr sx))
+	   (eqv? '@ (caadr sx)))
+      (cdadr sx)
+      '()))
+
+;; @deffn {Procedure} sx-attr-ref sx|node|tail key => val
+;; Return an attribute value given the key, or @code{#f}.
+;; Also works if passed the attribute node @code{(@ ...)} or its tail.
+;; @end deffn
+(define (sx-attr-ref sx key)
+  (let ((attr-tail (cond ((null? sx) sx)
+			 ((pair? (car sx)) sx)
+			 ((eqv? '@ (car sx)) (car sx))
+			 ((sx-attr sx))
+			 (else '()))))
+    (and=> (assq-ref attr-tail key) car)))
+
+;; @deffn {Procedure} sx-attr-add sx key-or-pair [val]
+;; Add attribute to sx, passing either a key-val pair or key and val.
+;; @end deffn
+(define* (sx-attr-add sx key-or-pair #:optional val)
+  (let* ((pair (if val (list key-or-pair val) key-or-pair))
+	 (key (car pair)) (val (cadr pair)))
+    (cons
+     (sx-tag sx)
+     (if (sx-has-attr? sx)
+	 (cons `(@ pair
+		   ,@(let iter ((atl (sx-attr sx)))
+		       (cond ((null? atl) '())
+			     ((eq? key (caar atl)) (iter (cdr atl)))
+			     (else (cons (car atl) (iter (cdr atl)))))))
+	       (cddr sx))
+	 (cons `(@ ,pair) (cdr sx))))))
+
+;; @deffn {Procedure} sx-attr-add* sx key val [key val [@dots{} ]] => sx
+;; Add key-val pairs. @var{key} must be a symbol and @var{val} must be
+;; a string.  Return a new @emph{sx}.
+;; @end deffn
+(define (sx-attr-add* sx . rest)
+  (let* ((attrs (sx-attr sx))
+	 (attrs (let iter ((kvl rest))
+		  (if (null? kvl) attrs
+		      (cons (list (car kvl) (cadr kvl)) (iter (cddr kvl)))))))
+    (cons* (sx-tag sx) (cons '@ attrs)
+	   (if (sx-has-attr? sx) (cddr sx) (cdr sx)))))
+
+;; @deffn {Procedure} sx-attr-set! sx key val
+;; Set attribute for sx.  If no attributes exist, if key does not exist,
+;; add it, if it does exist, replace it.
+;; @end deffn
+(define (sx-attr-set! sx key val)
+  (if (sx-has-attr? sx)
+      (let ((attr (cadr sx)))
+	(set-cdr! attr (assoc-set! (cdr attr) key (list val))))
+      (set-cdr! sx (cons `(@ (,key ,val)) (cdr sx))))
+  sx)
+
+;; @deffn {Procedure} sx-cons* tag attr ... => sx
+;; @deffnx {Procedure} sx-list tag attr ... => sx
+;; Generate the tag and the attr list if it exists.  Note that
+;; The following are equivalent:
+;; @example
+;; (sx-cons* tag attr elt1 elt2 '())
+;; (sx-list tag attr elt1 elt2)
+;; @end example
+;; @noindent
+;; @end deffn
+(define (sx-cons* tag attr . rest)
+  (if (null? rest) (error "expecing tail"))
+  (let ((attr (cond
+	       ((not attr) #f)
+	       ((null? attr) #f)
+	       ((pair? (car attr)) `(@ . ,attr))
+	       (else attr)))
+	(tail (let loop ((items rest))
+		(if (null? (cdr items)) (car items)
+		    (if (car items)
+			(cons (car items) (loop (cdr items)))
+			(loop (cdr items)))))))
+    (if attr (cons* tag attr tail) (cons tag tail))))
+(define (sx-list tag attr . rest)
+  (let ((attr (cond
+	       ((not attr) #f)
+	       ((null? attr) #f)
+	       ((pair? (car attr)) `(@ . ,attr))
+	       (else attr)))
+	(tail (let loop ((items rest))
+		(if (null? items) '()
+		    (if (car items)
+			(cons (car items) (loop (cdr items)))
+			(loop (cdr items)))))))
+    (if attr (cons* tag attr tail) (cons tag tail))))
+
+;; @deffn {Procedure} sx-split sexp => tag attr tail
+;; @deffnx {Procedure} sx-split* sexp => tag attr exp ...
+;; Split an SXML element into its constituent parts, as a @code{values},
+;; where @var{attr} is list of pairs.  If no attributes exist, @var{attr}
+;; is @code{'()}.
 ;; @end deffn
 (define (sx-split sexp)
   (let ((tag (sx-tag sexp))
-	 (attr (sx-attr sexp))
-	 (tail (sx-tail sexp)))
+	(attr (sx-attr sexp))
+	(tail (sx-tail sexp)))
     (values tag attr tail)))
 (define (sx-split* sexp)
   (let ((tag (sx-tag sexp))
-	 (attr (sx-attr sexp))
-	 (tail (sx-tail sexp)))
+	(attr (sx-attr sexp))
+	(tail (sx-tail sexp)))
     (apply values tag attr tail)))
 
-;; @deffn {Procedure} sx-join tag attr|#f tail => sexp
-;; @deffnx {Procedure} sx-join* tag attr|#f exp ... => sexp
-;; Build an SXML element by its parts.  If @var{ATTR} is @code{#f} skip;
+;; @deffn {Procedure} sx-join tag attr tail => sexp
+;; @deffnx {Procedure} sx-join* tag attr exp ... => sexp
+;; Build an SXML element by its parts.  If @var{ATTR} is @code{'()} skip;
+;; @code{sx-join*} will remove any exp that are @code{#f} or @code{'()}.
 ;; @end deffn
 (define (sx-join tag attr tail)
-  (if attr (cons* tag attr tail) (cons tag tail)))
+  (if (and attr (pair? attr))
+      (if (pair? (car attr))
+	  (cons* tag `(@ . ,attr) tail)
+	  (cons* tag `(@ ,attr) tail))
+      (cons tag tail)))
 (define (sx-join* tag attr . tail)
-  (if attr (cons* tag attr tail) (cons tag tail)))
+  (let ((tail (let loop ((tail tail))
+		(cond
+		 ((null? tail) '())
+		 ((not tail) (loop (cdr tail)))
+		 ((null? (car tail)) (loop (cdr tail)))
+		 (else (cons (car tail) (loop (cdr tail))))))))
+    (if (and attr (pair? attr))
+	(if (pair? (car attr))
+	    (cons* tag `(@ . ,attr) tail)
+	    (cons* tag `(@ ,attr) tail))
+	(cons tag tail))))
 
 ;; ============================================================================
 
@@ -422,16 +428,4 @@
     ;;^-- If not pair or unquote then must be string, right?
    ))
 
-;;; deprecated:
-
-(define sx-set-attr! sx-attr-set!)
-
-(define (sx-attr-set* sx . rest)
-  (let iter ((attr (or (and=> (sx-attr sx) cdr) '())) (kvl rest))
-    (cond
-     ((null? kvl) (cons* (sx-tag sx) (cons '@ (reverse attr)) (sx-tail sx 1)))
-     (else (iter (cons (list (car kvl) (cadr kvl)) attr) (cddr kvl))))))
-
-(define sx-set-attr* sx-attr-set*)
-
 ;; --- last line ---
