@@ -32,7 +32,7 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-37))
 
-(define *ffi-help-version* "0.86.3")
+(define *ffi-help-version* "0.86.4")
 
 (define %summary
   "Compile a ffi-file (C interface spec) to Scheme (or maybe .go).")
@@ -218,7 +218,7 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
       (sfmt "compiling `~A' ..." scm-file)
        (let ((go-file (compile-file scm-file
 				   #:from 'scheme #:to 'bytecode
-				   #:opts '((optimizations)))))
+				   #:opts '())))
 	(load-compiled go-file)
 	(sfmt " wrote `~A'\n" go-file))
       (sleep 1))))
